
/** Libreria que hace las operaciones matemáticas*/
#ifndef __LIBLINK__
#define __LIBLINK__

#include <iostream>
#include <Eigen/Dense>
#include <Eigen/SVD>
#include <math.h>
#include <string>

using namespace Eigen;
using namespace std;

#define lE1 0.071377 
#define lE2 0.145713
#define lE3 0.17947
#define lE4 0.08344

#define THRESHOLD 0.005
#define JDelta 0.000001
#define N_ITERACIONES 100
/** Matriz de transformacion entre el origen (coodenadas [0,0]) y el segundo "Joint" (servo) del brazo robotico. 
@param t
*/
Matrix4f getMat02(float t);
/** Matriz de transformacion entre el segundo y el tercer "Joint" (servo) del brazo robotico.
@param t
*/
Matrix4f getMat23(float t);
/** Matriz de transformacion entre el tecer y el cuarto "Joint" (servo) del brazo robotico.
@param t
 */
Matrix4f getMat34(float t);
/** Matriz de transformacion entre el cuarto "Joint" (servo) del brazo robotico y el punto final en medio del Gripper.
@param t
 */
Matrix4f getMat4f(float t);
/** Vector con la posicion final que alcanza el robot dado un conjunto de angulos q de los "Joint" (servos).
@param q
*/
Vector4f T(Vector4f q);				/* Final pos */
/** Calcula el jacobiano de T dado un conjunto de angulos q para los "Joint" (servos). 
@param q
*/
Matrix4f jacobian(Vector4f q);
/** Encuentra el conjutnos de angulos q necesarios para que el brazo llegue a la posicion T_final. q0 (angulos iniciales)puede ayudar a simplifacar los calculos si el resultado q es cercano a q0.
@param T_final
@param q0
*/
Vector4f solveQ(Vector4f T_final,Vector4f q0 = Vector4f(0,0,0,0));

#endif
