

#ifndef __LIBLINK_H__
#define __LIBLINK_H__
#include "libLink.hpp"
#endif

Matrix4f getMat02(float t)
{
	Matrix4f mat;
	mat<< 	cos(t), -sin(t), 0,  0,
			sin(t), cos(t),  0,  0,
			0,        0,     1,  lE1,
			0,        0,     0,  1;
	return mat;
}

Matrix4f getMat23(float t)
{
	Matrix4f mat;
	mat<< 	cos(t),		0,	sin(t),		lE2*sin(t),
			0,			1,	0,			0,
			-sin(t),	0,	cos(t),		lE2*cos(t),
			0,			0,	0,			1;
	return mat;
}

Matrix4f getMat34(float t)
{
	Matrix4f mat;
	mat<< 	cos(t),		0,	sin(t),		lE3*sin(t),
			0,			1,	0,			0,
			-sin(t),	0,	cos(t),		lE3*cos(t),
			0,			0,	0,			1;
	return mat;
}

Matrix4f getMat4f(float t)
{
	Matrix4f mat;
	mat<< 	cos(t),		0,	sin(t),		lE4*sin(t),
			0,			1,	0,			0,
			-sin(t),	0,	cos(t),		lE4*cos(t),
			0,			0,	0,			1;
	return mat;
}

Vector4f T(Vector4f q)
{
	Vector4f origen(0,0,0,1);
	return getMat02(q(0))*getMat23(q(1))*getMat34(q(2))*getMat4f(q(3))*origen;
}


Matrix4f jacobian(Vector4f q)
{
	Vector4f dT_dt1, dT_dt2, dT_dt3, dT_dt4;

	dT_dt1 = (T(Vector4f(q(0)+JDelta,q(1),q(2),q(3)))-T(q)) / JDelta;
	dT_dt2 = (T(Vector4f(q(0),q(1)+JDelta,q(2),q(3)))-T(q)) / JDelta;
	dT_dt3 = (T(Vector4f(q(0),q(1),q(2)+JDelta,q(3)))-T(q)) / JDelta;
	dT_dt4 = (T(Vector4f(q(0),q(1),q(2),q(3)+JDelta))-T(q)) / JDelta;

	Matrix4f jacobian_return;
	jacobian_return << 	dT_dt1(0),	dT_dt2(0),	dT_dt3(0),	dT_dt4(0),
						dT_dt1(1),	dT_dt2(1),	dT_dt3(1),	dT_dt4(1),
						dT_dt1(2),	dT_dt2(2),	dT_dt3(2),	dT_dt4(2),
						dT_dt1(3),	dT_dt2(3),	dT_dt3(3),	dT_dt4(3);


	return jacobian_return;
}

Vector4f solveQ(Vector4f T_final,Vector4f q0)
{
	float dist = 10.0;
	int cont = 0;
	Vector4f T0;

	while(dist > pow(THRESHOLD,2) && cont<N_ITERACIONES)
	{
		T0 = T(q0);
		Vector4f dt = T_final - T0;
		Matrix4f J = jacobian(q0);
		JacobiSVD<MatrixXf> svd(J,ComputeThinU | ComputeThinV);
		Vector4f dq = svd.solve(dt);
		q0 = dq + q0;

		cont++;
		dist = pow(T_final(0)-T0(0),2)+pow(T_final(1)-T0(1),2)+pow(T_final(2)-T0(2),2);
	}
	return q0;

}




















