#include "mainwindow.h"
#include "ui_mainwindow.h"

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    QPixmap iconS(":start.png");
    QPixmap iconSt(":stop.png");
    QPixmap iconR(":run.png");
    QPixmap iconC(":sync.png");
    QPixmap iconP(":pause.png");
    QPixmap iconGT(":getto.png");
    QPixmap logo(":logo.png");

    setWindowIcon(logo);

    ui->button_start->setIcon(iconS);
    ui->button_connect->setIcon(iconC);
    ui->button_get->setIcon(iconGT);
    ui->button_pause->setIcon(iconP);
    ui->button_run->setIcon(iconR);
    ui->button_stop->setIcon(iconSt);

    sim.addJoint(Joint(0,"link0"));
    sim.addJoint(Joint(1,"link1"));
    sim.addJoint(Joint(2,"link2"));
    sim.addJoint(Joint(3,"link3"));
    sim.addJoint(Joint(4,"j_g1"));
    sim.addJoint(Joint(5,"j_g2"));

    Q0 = Vector4f(0,0,0,1);

    connect(ui->button_start,SIGNAL(clicked(bool)),this,SLOT(start(bool)));
    connect(ui->button_connect,SIGNAL(clicked(bool)),this,SLOT(createNewSim(bool)));
    connect(ui->button_pause,SIGNAL(clicked(bool)),this,SLOT(pause(bool)));
    connect(ui->button_stop,SIGNAL(clicked(bool)),this,SLOT(stop(bool)));
    connect(ui->button_get,SIGNAL(clicked(bool)),this,SLOT(goTo(bool)));
    connect(ui->button_run,SIGNAL(clicked(bool)),this,SLOT(runCode(bool)));

    if(sim.getId() == -1)
    {
        ui->displayMsg->setText(QString("Not connected xd"));
    }
}

MainWindow::~MainWindow()
{
    delete ui;
}


void MainWindow::start(bool)
{
    sim.start();
}

void MainWindow::createNewSim(bool)
{
    sim = MinimalVrep();
    if(sim.getId() == -1)
    {
        ui->displayMsg->setText(QString("Conection refused :("));
    }
}

void MainWindow::pause(bool)
{
    sim.pause();
}
void MainWindow::stop(bool)
{
    sim.stop();
}

void MainWindow::goTo(bool)
{
    float x = (ui->x_pos->text()).toFloat();
    float y = (ui->y_pos->text()).toFloat();
    float z = (ui->z_pos->text()).toFloat();

    moveTo(x,y,z);
}

void MainWindow::moveTo(float x,float y, float z)
{
    if(x < 0.001){
        x = 0.001;      /* Fixing a math bug */
    }
    Vector4f T_Final(x,y,z,1);
    Vector4f Q = solveQ(T_Final);

    Q0 = Q;

    ui->displayMsg->setText(QString("q0: %1   q1: %2   q2:%3     q3: %4   :D").arg(fmod(Q(0),2*M_PI)).arg(fmod(Q(1),2*M_PI)).arg(fmod(Q(2),2*M_PI)).arg(fmod(Q(3),2*M_PI)));

    sim.setJointPos(0,fmod(Q(0),2*M_PI));
    usleep(100000);
    sim.setJointPos(1,fmod(Q(1),2*M_PI));
    sim.setJointPos(2,fmod(Q(2),2*M_PI));
    sim.setJointPos(3,fmod(Q(3),2*M_PI));

}

void MainWindow::runCode(bool)
{
    QStringList commands = (ui->commandBox->toPlainText()).split("\n");
    QStringList sub_commands;
    float x,y,z,delay;

    for(int i=0 ; i<commands.size() ; i++)
    {
        sub_commands = commands.at(i).split(" ");
        if(sub_commands.size() == 4)
        {
            x = sub_commands.at(0).toFloat();
            y = sub_commands.at(1).toFloat();
            z = sub_commands.at(2).toFloat();
            delay = sub_commands.at(3).toFloat();
            moveTo(x,y,z);
            usleep(delay*1000);
        }
        else
        {
            ui->displayMsg->setText(QString("Error: Format a line %1. Must be: x y z delay").arg(i+1));
        }
    }
}
