#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include "MinimalVrep.hpp"
#include "libLInK/libLink.hpp"
#include <vector>
#include <QDebug>
#include <QPointF>
#include <unistd.h>

namespace Ui {
/** Clase que contiene la interfaz visual de Link (frame)*/    
class MainWindow;
}

class MainWindow : public QMainWindow
{
    Q_OBJECT
//Metodos publicos
public:
/** Constructor*/
    explicit MainWindow(QWidget *parent = 0);
/** Destructor*/
    ~MainWindow();
/** Mueve la simulacion a la posicion (x,y,z) especificada. 
@param x
@param y
@param z
*/    
    void moveTo(float x, float y, float z);
public slots:
/** Inicia/reanuda la simulacion en Vrep. */
    void start(bool);
/** Pausa la simulacion en Vrep. */
    void pause(bool);
/** Detiene la comunicacion en Vrep. Se vuelve a los valores por defecto. */
    void stop(bool);
/** Se llama a moveTo con los valores especificados en las entradas de la esquina superior izquierda. */
    void goTo(bool);
/** Se llama a moveTo con la secuencia de valores especificados en la caja de comandos. */
    void runCode(bool);
/** Intenta una nueva conexion con Vrep. */
    void createNewSim(bool);

//Atributos Privados
private:
/** Interfaz grafica de Qt. */
    Ui::MainWindow *ui;
/** Controlador de la simulacion en Vrep */
    MinimalVrep sim;
/** Valor actual de los angulos usado para simplicar calculos (en esta implementacion no se utiliza). */
    Vector4f Q0;
};

#endif // MAINWINDOW_H
