

#include <vector>
#include <string>
#include <iostream>

extern "C"{
	#include "extApi.h"
}

#define PORT 19997

using namespace std;

/** Objeto basico que representa un "Joint" (servo) en la simulacion de Vrep. En nombre del objeto Joint debe coincidir con el nombre del Joint en Vrep*/
class Joint
{
		//Metodos Publicos
public:
/** Constructor 
@param id_
@param name_
*/
	Joint(int id_,string name_)
	{
		id =id_;
		name = name_;

	}
	const char *getName()
	{
		return name.c_str();
	}
//Atributos Privados
private:
/** Nombre del Joint. Debe coincidir con el nombre del Joint en Vrep.*/
	string name;
/** No debe repetirse con el id_ de otros objetos Joint.*/
	int id;
};

/**Clase que interactua API*/
class MinimalVrep
{
	//Metodos Publicos
public:
/** Constructor */
	MinimalVrep();
/** Inicia/reanuda la simulacion. */	
	void start();
/** Pausa la simulacion.*/	
	void pause();
/** Detiene la simula. Se restauran los valores por defecto.*/	
	void stop();
/** Permite añadir un "joint" (servo) al conjunto de servos controlados por la simulacion.*/		
	void addJoint(Joint j);
/** Mueve un determinado "joint" (servo) al angulo indicado en pos.*/		
	void setJointPos(int indice, float pos);
/** Obtiene el angulo actual de un determinado "joint" (servo).*/		
	float getJointPos(int indice);
/** */	 
	int getId();
//Atributos Privados
private:
/** Vector de "joints" (servos) que pueden ser controlados por el programa.*/
	vector<Joint> joints;
/** Identificador de la comunicacion via sockets con Vrep.*/	
	int com_id;
};
